package speiger.src.coreengine.math.vector.bytes;

import java.nio.ByteBuffer;

import speiger.src.coreengine.math.MathUtils;
import speiger.src.coreengine.math.vector.doubles.Vec3d;
import speiger.src.coreengine.math.vector.floats.Vec3f;
import speiger.src.coreengine.math.vector.ints.Vec3i;
import speiger.src.coreengine.math.vector.longs.Vec3l;
import speiger.src.coreengine.math.vector.shorts.Vec3s;

public interface Vec3b extends Vecb {
	public static final Vec3b ZERO = of();
	public static final Vec3b MINUS_ONE = of((byte)-1);
	public static final Vec3b ONE = of((byte)1);
	
	public static Vec3b mutable() { return new Vec3bMutable(); }
	public static Vec3b mutable(byte value) { return new Vec3bMutable(value); }
	public static Vec3b mutable(byte x, byte y, byte z) { return new Vec3bMutable(x, y, z); }
	public static Vec3b mutable(Vec3b vec) { return mutable(vec.x(), vec.y(), vec.z()); }
	public static Vec3b of() { return new Vec3bImmutable(); }
	public static Vec3b of(byte value) { return new Vec3bImmutable(value); }
	public static Vec3b of(byte x, byte y, byte z) { return new Vec3bImmutable(x, y, z); }
	public static Vec3b of(Vec3b vec) { return of(vec.x(), vec.y(), vec.z()); }
	
	public byte x();
	public byte y();
	public byte z();
	public Vec3b x(byte x);
	public Vec3b y(byte y);
	public Vec3b z(byte z);
	
	@Override
	public default byte[] asArray() { return new byte[] {x(), y(), z()}; }
	@Override
	public Vec3b copy();
	@Override
	public default Vec3b abs() { return set((byte)Math.abs(x()), (byte)Math.abs(y()), (byte)Math.abs(z())); }
	@Override
	public default Vec3b negate() { return set((byte)0, (byte)0, (byte)0); }
	@Override
	public default Vec3b invert() { return set((byte)-x(), (byte)-y(), (byte)-z()); }
	@Override
	public default Vec3b add(byte value) { return add(value, value, value); }
	public default Vec3b add(Vec3b value) { return add(value.x(), value.y(), value.z()); }
	public default Vec3b add(byte x, byte y, byte z) { return set((byte)(x() + x), (byte)(y() + y), (byte)(z() + z)); }
	@Override
	public default Vec3b sub(byte value) { return sub(value, value, value); }
	public default Vec3b sub(Vec3b value) { return sub(value.x(), value.y(), value.z()); }
	public default Vec3b sub(byte x, byte y, byte z) { return set((byte)(x() - x), (byte)(y() - y), (byte)(z() - z)); }
	@Override
	public default Vec3b multiply(byte value) { return multiply(value, value, value); }
	public default Vec3b multiply(Vec3b value) { return multiply(value.x(), value.y(), value.z()); }
	public default Vec3b multiply(byte x, byte y, byte z) { return set((byte)(x() * x), (byte)(y() * y), (byte)(z() * z)); }
	@Override
	public default Vec3b devide(byte value) { return devide(value, value, value); }
	public default Vec3b devide(Vec3b value) { return devide(value.x(), value.y(), value.z()); }
	public default Vec3b devide(byte x, byte y, byte z) { return set((byte)(x() / x), (byte)(y() / y), (byte)(z() / z)); }
	@Override
	public default Vec3b set(byte value) { return set(value, value, value); }
	public default Vec3b set(Vec3b value) { return set(value.x(), value.y(), value.z()); }
	public Vec3b set(byte x, byte y, byte z);
	public default double distanceTo(Vec3b value) { return distanceTo(value.x(), value.y(), value.z()); }
	public default double distanceTo(byte x, byte y, byte z) { return Math.sqrt(distanceToSquared(x, y, z)); }
	public default long distanceToSquared(Vec3b value) { return distanceToSquared(value.x(), value.y(), value.z()); }
	
	public default long distanceToSquared(byte x, byte y, byte z) {
		long xPos = x() - x;
		long yPos = y() - y;
		long zPos = z() - z;
		return (xPos * xPos) + (yPos * yPos) + (zPos * zPos);
	}
	
	@Override
	public default long lengthSquared() { return (x() * x()) + (y() * y()) + (z() * z()); }
	public default long dotProduct(Vec3b value) { return dotProduct(value.x(), value.y(), value.z()); }
	public default long dotProduct(byte x, byte y, byte z) { return (x() * x) + (y() * y) + (z() * z); }
	public default Vec3b min(Vec3b other) { return min(other, this); }
	public default Vec3b min(Vec3b other, Vec3b result) { return min(other.x(), other.y(), other.z(), result); }
	public default Vec3b min(byte x, byte y, byte z) { return min(x, y, z, this); }
	public default Vec3b min(byte x, byte y, byte z, Vec3b result) { return result.set((byte)Math.min(x(), x), (byte)Math.min(y(), y), (byte)Math.min(z(), z)); }
	public default Vec3b max(Vec3b other) { return max(other, this); }
	public default Vec3b max(Vec3b other, Vec3b result) { return max(other.x(), other.y(), other.z(), result); }
	public default Vec3b max(byte x, byte y, byte z) { return max(x, y, z, this); }
	public default Vec3b max(byte x, byte y, byte z, Vec3b result) { return result.set((byte)Math.max(x(), x), (byte)Math.max(y(), y), (byte)Math.max(z(), z)); }
	public default Vec3b difference(Vec3b other) { return difference(other, this); }
	public default Vec3b difference(Vec3b other, Vec3b result) { return difference(other.x(), other.y(), other.z(), result); }
	public default Vec3b difference(byte x, byte y, byte z) { return difference(x, y, z, this); }
	public default Vec3b difference(byte x, byte y, byte z, Vec3b result) { return result.set((byte)(x() - x), (byte)(y() - y), (byte)(z() - z)); }
	@Override
	public default Vec3b clamp(byte min, byte max) { return clamp(min, max, ALL); }
	public default Vec3b clamp(byte min, byte max, Vec3b result) { return clamp(min, max, result, ALL); }
	@Override
	public default Vec3b clamp(byte min, byte max, int filter) { return clamp(min, max, this, filter); }
	public default Vec3b clamp(byte min, byte max, Vec3b result, int filter) { return result.set((filter & X) == 0 ? x() : MathUtils.clamp(min, max, x()), (filter & Y) == 0 ? y() : MathUtils.clamp(min, max, y()), (filter & Z) == 0 ? z() : MathUtils.clamp(min, max, z())); }
	
	@Override
	public default Vec3b store(ByteBuffer buffer) {
		buffer.put(x()).put(y()).put(z());
		return this;
	}
	
	@Override
	public default Vec3b load(ByteBuffer buffer) { return set(buffer.get(), buffer.get(), buffer.get()); }
	@Override
	public default Vec3s asShort() { return isMutable() ? Vec3s.mutable(x(), y(), z()) : Vec3s.of(x(), y(), z()); }
	@Override
	public default Vec3i asInt() { return isMutable() ? Vec3i.mutable(x(), y(), z()) : Vec3i.of(x(), y(), z()); }
	@Override
	public default Vec3l asLong() { return isMutable() ? Vec3l.mutable(x(), y(), z()) : Vec3l.of(x(), y(), z()); }
	@Override
	public default Vec3f asFloat() { return isMutable() ? Vec3f.mutable(x(), y(), z()) : Vec3f.of(x(), y(), z()); }
	@Override
	public default Vec3d asDouble() { return isMutable() ? Vec3d.mutable(x(), y(), z()) : Vec3d.of(x(), y(), z()); }
	@Override
	public default Vec3b asMutable() { return isMutable() ? this : mutable(this); }
	@Override
	public default Vec3b asImmutable() { return isMutable() ? of(this) : this; }
	@Override
	public default Vec3b copyAsMutable() { return mutable(this); }
	@Override
	public default Vec3b copyAsImmutable() { return of(this); }
}
